/* This file is part of the KDE project
   Copyright 2007,2008 Will Stephenson <wstephenson@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor approved
   by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "facebookresource.h"

#include <QtDBus/QDBusMessage>
#include <KDebug>
#include <KDialog>
#include <KWindowSystem>

#include <kabc/addressee.h>

#include <kfacebook/method.h>
#include <kfacebook/version.h>
#include <akonadi/itemcreatejob.h>
#include <akonadi/session.h>

FacebookResource::FacebookResource( const QString & id ) : Akonadi::ResourceBase( id ), m_session( 0 ), m_dialog( 0 )
{
  m_session = new KFacebook::Session( this, KFACEBOOK_API_KEY, KFACEBOOK_SECRET_KEY );
}

FacebookResource::~FacebookResource()
{

}

void FacebookResource::configure( WId windowId )
{
    kDebug() << endl;
    if ( m_dialog == 0 ) {
        m_dialog = new KDialog;
        m_ui.setupUi( m_dialog->mainWidget() );
        //connect( m_dialog, SIGNAL(okClicked()), this, SLOT(configAccepted()) );
        if ( m_session->state() != KFacebook::Session::Authenticated ) {
          m_ui.btnNewSession->setText( i18n( "Click when authenticated" ) );
          connect( m_ui.btnNewSession, SIGNAL( clicked() ), this, SLOT( browserLoggedIn() ) );
        }
        else
          connect( m_ui.btnNewSession, SIGNAL( clicked() ), this, SLOT( createNewSession() ) );
    }
    if ( windowId != 0 )
        KWindowSystem::setMainWindow( m_dialog, windowId );
    m_dialog->show();
}

void FacebookResource::browserLoggedIn()
{
  m_session->browserLoggedIn();
}

void FacebookResource::setupCollection()
{
  if (m_session->state() == KFacebook::Session::Authenticated && !m_collection.isValid() ) {
    m_collection.setParent( Akonadi::Collection::root() );
    m_collection.setRemoteId( m_session->uid() );
    m_collection.setName( QString("Friends of %1" ).arg( m_session->uid() ) );
    QStringList contentTypes;
    contentTypes << "text/vcard";
    m_collection.setContentMimeTypes( contentTypes );
  } else {
    kDebug() << "failed to setup collection!" << endl;
  }
}

void FacebookResource::retrieveCollections()
{
  kDebug() << endl;
  setupCollection();
  Akonadi::Collection::List list;
  list.append( m_collection );
  collectionsRetrieved( list );
}

void FacebookResource::retrieveItems( const Akonadi::Collection & collection )
{
  kDebug() << endl;
  //if ( collection == m_collection ) {
    KFacebook::Job * job = m_session->friendsGetJob();
    connect( job, SIGNAL( result( KJob * ) ), SLOT( gotFriends( KJob * ) ) );
  //}
}

void FacebookResource::gotFriends( KJob * job )
{
  kDebug() << endl;
  KFacebook::FriendsGetJob * gfJob  = static_cast< KFacebook::FriendsGetJob * >( job );
  UidList friends = gfJob->friends();
  foreach ( uint uid, friends ) {
    kDebug() << "Adding Item '" << uid << "'" << endl;
    Akonadi::Item item;
    item.setRemoteId(  QString::number( uid ) );
    item.setMimeType( "text/vcard" );
    new Akonadi::ItemCreateJob( item, currentCollection() );
  }
  itemsRetrievalDone();
}

bool FacebookResource::retrieveItem( const Akonadi::Item & item, const QSet<QByteArray> & parts )
{
  const QString remoteId = item.remoteId();
  kDebug() << remoteId << ", " << parts << endl;

  QStringList wantedFields;
  wantedFields << KFacebook::Method::FIELD_NAME << KFacebook::Method::FIELD_FIRST_NAME << KFacebook::Method::FIELD_LAST_NAME << KFacebook::Method::FIELD_BIRTHDAY << KFacebook::Method::FIELD_ABOUT_ME << KFacebook::Method::FIELD_PICTURE_SMALL;
  // get the data for the requested users and update the Data
  QList<uint> uins;
  uins.append( remoteId.toUInt() );
  KFacebook::Job * job = m_session->usersGetInfoJob( uins, wantedFields );
  connect( job, SIGNAL( result( KJob * ) ), SLOT( gotUserInfo( KJob * ) ) );
  return true;
}

void FacebookResource::gotUserInfo( KJob * job )
{
  KFacebook::UsersGetInfoJob * uiJob = static_cast< KFacebook::UsersGetInfoJob * >( job );
  QList<KFacebook::UserInfo> userInfoList = uiJob->userInfo();
  KFacebook::UserInfo info = userInfoList.first();

  KABC::Addressee addressee;
  addressee.setName( info.name() );
  addressee.setGivenName( info.firstName() );
  addressee.setFamilyName( info.lastName() );
  //addressee.setBirthday( ); // no guaranteed formatting, therefore useless
  addressee.setNote( info.aboutMe() );
  KABC::Picture pic( info.smallPicture().url() );
  addressee.setPhoto( pic );
  Akonadi::Item item = currentItem();
  item.setMimeType( "text/vcard" );
  item.setPayload<KABC::Addressee>( addressee );
  itemRetrieved( item );
}

AKONADI_RESOURCE_MAIN( FacebookResource )

#include "facebookresource.moc"
