/*  This file is part of the KDE project
    Copyright (C) 2007,2008 Will Stephenson <wstephenson@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library.  If not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.

*/

#include <kdebug.h>
#include <kio/jobclasses.h>

#include "jobs.h"
#include "method.h"
#include "signature.h"
#include "userinfoxmlhelper.h"
#include "version.h"

#include "fbjobs.h"

UidList parseUidList( const QDomElement & elem );

KFacebook::Venue parseVenue( const QDomElement & elem );

namespace KFacebook
{

class KFacebook::Job::JobPrivate
{
public:
    KFacebook::Error error;
    QDomDocument response;
    QByteArray rawResponse;
};

} // namespace KFacebook

KFacebook::Job::Job( const KUrl & url ) : KIO::SpecialJob( url, QByteArray() ), d( new KFacebook::Job::JobPrivate )
{
    QByteArray args;
    QDataStream stream( &args, QIODevice::WriteOnly );
    stream << (int) 1 << url;
    setArguments( args );
    addMetaData( "accept", "text/xml" );
    QString userAgent = QString( "KDE Facebook Client %1" ).arg( "1.0" );
    addMetaData( "UserAgent", userAgent );
}

KFacebook::Job::~Job()
{
    delete d;
}

void KFacebook::Job::slotFinished()
{
    kDebug() << QString::fromUtf8(d->rawResponse) << endl;
    if ( !error() ) {
        if ( d->response.setContent( d->rawResponse, true ) ) {
            // look for an error and parse it
            QDomElement docElem = d->response.documentElement();
            if ( docElem.tagName() == "error_response" ) {
                QDomNode n = docElem.firstChild();
                while (!n.isNull()) {
                    QDomElement e = n.toElement(); // try to convert the node to an element.
                    if (!e.isNull()) {
                        if ( e.tagName() == "error_code" ) {
                            d->error = KFacebook::Error( e.text().toInt() );
                        }
                    }
                    n = n.nextSibling();
                }
            }
        } else {
            d->error = KFacebook::Error( - 1 );
        }
    }

    processResponse();

    TransferJob::slotFinished();
}

void KFacebook::Job::slotData( const QByteArray & data )
{
    unsigned int oldSize = d->rawResponse.size();
    d->rawResponse.resize( oldSize + data.size() );
    memcpy( d->rawResponse.data() + oldSize, data.data(), data.size() );
}

KFacebook::Error KFacebook::Job::platformError() const
{
    return d->error;
}

QDomDocument KFacebook::Job::response() const
{
    return d->response;
}

/***************************************************************************/

KFacebook::AuthCreateTokenJob::AuthCreateTokenJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::AuthCreateTokenJob::processResponse()
{
    QDomDocument doc = response();
    QDomElement docElem = doc.documentElement();
    m_token = docElem.text();
}

KFacebook::AuthGetSessionJob::AuthGetSessionJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::AuthGetSessionJob::processResponse()
{

    kDebug() << "RESPONSE: " << response().toString() << endl;
    QDomDocument doc = response();
    QDomElement docElem = doc.documentElement();
    QDomNode n = docElem.firstChild();
    while (!n.isNull()) {
        QDomElement e = n.toElement();
        if (!e.isNull()) {
            if ( e.tagName() == "session_key" ) {
                m_sessionKey = e.text();
            } else if ( e.tagName() == "uid" ) {
                m_uid = e.text();
            } else if ( e.tagName() == "expires" ) {
                m_expires.setTime_t( e.text().toUInt() );
            } else if ( e.tagName() == "secret" ) {
                m_secret = e.text();
            }
        }
        n = n.nextSibling();
    }
}

KFacebook::AuthLogoutJob::AuthLogoutJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::AuthLogoutJob::processResponse()
{

}

KFacebook::FriendsGetJob::FriendsGetJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::FriendsGetJob::processResponse()
{
    QDomDocument doc = response();
    QDomElement docElem = doc.documentElement();
    QDomNode n = docElem.firstChild();
    while (!n.isNull()) {
        QDomElement e = n.toElement(); // try to convert the node to an element.
        if (!e.isNull()) {
            if ( e.tagName() == "uid" ) {
                m_friends.append( e.text().toUInt() );
            }
        }
        n = n.nextSibling();
    }
}

KFacebook::FriendsAreFriendsJob::FriendsAreFriendsJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::FriendsAreFriendsJob::processResponse()
{
    QDomDocument doc = response();
    QDomElement docElem = doc.documentElement();
    QDomNode n = docElem.firstChild();
    while (!n.isNull()) {
        QDomElement e = n.toElement(); // try to convert the node to an element.
        if (!e.isNull()) {
            if ( e.tagName() == "friend_info" ) {
                uint uid1, uid2;
                bool areFriends = false;
                QDomNode n1 = e.firstChild();
                while (!n1.isNull()) {
                    QDomElement e1 = n1.toElement(); // try to convert the node to an element.
                    if (!e1.isNull()) {
                        if ( e1.tagName() == "uid1" ) {
                            uid1 = e1.text().toInt();
                        } else if ( e1.tagName() == "uid2" ) {
                            uid2 = e1.text().toInt();
                        } else if ( e1.tagName() == "are_friends" ) {
                            if (e1.text() == "1" )
                                areFriends = true;
                        }

                    }
                    n1 = n1.nextSibling();
                }
                if ( areFriends ) {
                    m_friendsList.append( QPair<uint,uint>( uid1, uid2 ) );
                } else {
                    m_notFriendsList.append( QPair<uint,uint>( uid1, uid2 ) );
                }
            }
        }
        n = n.nextSibling();
    }
}

KFacebook::UsersGetInfoJob::UsersGetInfoJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::UsersGetInfoJob::processResponse()
{
    QDomDocument doc = response();
    QDomElement docElem = doc.documentElement();
    m_userInfo = UserInfoXMLHelper::parseUsers( docElem );
}

KFacebook::UsersGetLoggedInUserJob::UsersGetLoggedInUserJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::UsersGetLoggedInUserJob::processResponse()
{
    QDomDocument doc = response();
    QDomElement docElem = doc.documentElement();
    m_uid = docElem.text().toUInt();
}

KFacebook::FeedPublishJob::FeedPublishJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::FeedPublishJob::processResponse()
{
    QDomDocument doc = response();
    QDomElement docElem = doc.documentElement();
    m_success = (docElem.text() == "1");
}

KFacebook::FQLQueryJob::FQLQueryJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::FQLQueryJob::processResponse()
{

}

KFacebook::FriendsGetAppUsersJob::FriendsGetAppUsersJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::FriendsGetAppUsersJob::processResponse()
{
    QDomDocument doc = response();
    QDomElement docElem = doc.documentElement();
    QDomNode n = docElem.firstChild();
    while (!n.isNull()) {
        QDomElement e = n.toElement(); // try to convert the node to an element.
        if (!e.isNull()) {
            if ( e.tagName() == "uid" ) {
                m_appUsers.append( e.text().toUInt() );
            }
        }
        n = n.nextSibling();
    }
}

KFacebook::NotificationsGetJob::NotificationsGetJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::NotificationsGetJob::processResponse()
{
    kDebug() << endl;
    QDomDocument doc = response();
    QDomElement docElem = doc.documentElement();
    QDomNode n = docElem.firstChild();
    while (!n.isNull()) {
        QDomElement e = n.toElement(); // try to convert the node to an element.
        if (!e.isNull()) {
            if ( e.tagName() == "messages" ) {
                parseNotificationsStanza( e, m_unreadMsgs, m_lastMsg );
            } else if ( e.tagName() == "pokes" ) {
                parseNotificationsStanza( e, m_unreadPokes, m_lastPoke );
            } else if ( e.tagName() == "shares" ) {
                parseNotificationsStanza( e, m_unreadShares, m_lastShare );
            } else if ( e.tagName() == "friend_requests" ) {
                QDomNode n1 = e.firstChild();
                while (!n1.isNull()) {
                    QDomElement e1 = n1.toElement(); // try to convert the node to an element.
                    if (!e1.isNull()) {
                        if ( e1.tagName() == "uid" ) {
                            m_friendRequests.append( e1.text().toUInt() );
                        }
                    }
                    n1 = n.nextSibling();
                }
            } else if ( e.tagName() == "group_invites" ) {
                QDomNode n1 = e.firstChild();
                while (!n1.isNull()) {
                    QDomElement e1 = n1.toElement(); // try to convert the node to an element.
                    if (!e1.isNull()) {
                        if ( e1.tagName() == "gid" ) {
                            m_groupInvites.append( e1.text().toUInt() );
                        }
                    }
                    n1 = n.nextSibling();
                }
            } else if ( e.tagName() == "event_requests" ) {
                QDomNode n1 = e.firstChild();
                while (!n1.isNull()) {
                    QDomElement e1 = n1.toElement(); // try to convert the node to an element.
                    if (!e1.isNull()) {
                        if ( e1.tagName() == "eid" ) {
                            m_eventInvites.append( e1.text().toUInt() );
                        }
                    }
                    n1 = n1.nextSibling();
                }
            }
        }
        n = n.nextSibling();
    }
}

void KFacebook::NotificationsGetJob::parseNotificationsStanza( const QDomElement & elem, uint& unread, uint& last )
{
    QDomNode n = elem.firstChild();
    while (!n.isNull()) {
        QDomElement e = n.toElement(); // try to convert the node to an element.
        if (!e.isNull()) {
            if ( e.tagName() == "unread" ) {
                unread = e.text().toUInt();
            } else if ( e.tagName() == "most_recent" ) {
                last = e.text().toUInt();
            }
        }
        n = n.nextSibling();
    }
}

KFacebook::NotificationsSendJob::NotificationsSendJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::NotificationsSendJob::processResponse()
{
    QDomDocument doc = response();
    QDomElement docElem = doc.documentElement();
    m_url = KUrl( docElem.text() );
}

KFacebook::NotificationsSendRequestJob::NotificationsSendRequestJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::NotificationsSendRequestJob::processResponse()
{
    QDomDocument doc = response();
    QDomElement docElem = doc.documentElement();
    m_url = KUrl( docElem.text() );
}

KFacebook::ProfileSetFBMLJob::ProfileSetFBMLJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::ProfileSetFBMLJob::processResponse()
{
    QDomDocument doc = response();
    QDomElement docElem = doc.documentElement();
    m_success = (docElem.text() == "1");
}

KFacebook::ProfileGetFBMLJob::ProfileGetFBMLJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::ProfileGetFBMLJob::processResponse()
{
    QDomDocument doc = response();
    QDomElement docElem = doc.documentElement();
    m_fbml = docElem.text();
}

KFacebook::EventsGetJob::EventsGetJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::EventsGetJob::processResponse()
{
    uint eid, nid, creator;
    QString name, tagline, host, description, type, subType, location;
    KUrl smallPicUrl, medPicUrl, largePicUrl;
    QDateTime startTime, endTime, updateTime;
    Venue venue;

    QDomDocument doc = response();
    QDomElement docElem = doc.documentElement();
    QDomNode t = docElem.firstChild();
    while (!t.isNull()) {
        QDomElement f = t.toElement();

        if ( f.tagName() == "event" ) {
            QDomNode n = f.firstChild();
            while (!n.isNull()) {
                QDomElement e = n.toElement(); // try to convert the node to an element.
                if (!e.isNull()) {
                    QString tag = e.tagName();
                    if ( tag == "eid" ) {
                        eid = e.text().toUInt();
                    } else if ( tag == "name" ) {
                        name = e.text();
                    } else if ( tag == "tagline" ) {
                        tagline = e.text();
                    } else if ( tag == "nid" ) {
                        nid = e.text().toUInt();
                    } else if ( tag == "pic_small" ) {
                        smallPicUrl = KUrl( e.text() );
                    } else if ( tag == "pic" ) {
                        medPicUrl = KUrl( e.text() );
                    } else if ( tag == "pic_large" ) {
                        largePicUrl = KUrl( e.text() );
                    } else if ( tag == "host" ) {
                        host = e.text();
                    } else if ( tag == "description" ) {
                        description = e.text();
                    } else if ( tag == "event_type" ) {
                        type = e.text();
                    } else if ( tag == "event_subtype" ) {
                        subType = e.text();
                    } else if ( tag == "start_time" ) {
                        startTime.setTime_t( e.text().toUInt() );
                    } else if ( tag == "end_time" ) {
                        endTime.setTime_t( e.text().toUInt() );
                    } else if ( tag == "creator" ) {
                        creator = e.text().toUInt();
                    } else if ( tag == "update_time" ) {
                        updateTime.setTime_t( e.text().toUInt() );
                    } else if ( tag == "location" ) {
                        location = e.text();
                    } else if ( tag == "venue" ) {
                        venue = parseVenue( e );
                    }

                }
                n = n.nextSibling();
            }
            m_events.append( KFacebook::Event( eid, name, tagline, nid, smallPicUrl, medPicUrl, largePicUrl, host, description, type, subType, startTime, endTime, creator, updateTime, location, venue ) );
        }
        t = t.nextSibling();
    }
    foreach (KFacebook::Event evt, m_events ) {
        evt.dump();
    }
}

KFacebook::Venue parseVenue( const QDomElement & e )
{
    QString venStreet, venCity, venState, venCountry;
    QDomNode n1 = e.firstChild();
    while (!n1.isNull()) {
        QDomElement e1 = n1.toElement(); // try to convert the node to an element.
        if (!e1.isNull()) {
            if ( e1.tagName() == "street" ) {
                venStreet = e1.text();
            } else if ( e1.tagName() == "city" ) {
                venCity = e1.text();
            } else if ( e1.tagName() == "country" ) {
                venCountry = e1.text();
            } else if ( e1.tagName() == "state" ) {
                venState = e1.text();
            }
        }
        n1 = n1.nextSibling();
    }
    return KFacebook::Venue( venStreet, venCity, venStreet, venCountry );
}

KFacebook::EventsGetMembersJob::EventsGetMembersJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::EventsGetMembersJob::processResponse()
{
    QDomDocument doc = response();
    QDomElement docElem = doc.documentElement();
    QDomNode n = docElem.firstChild();
    while (!n.isNull()) {
        QDomElement e = n.toElement(); // try to convert the node to an element.
        if (!e.isNull()) {
            if ( e.tagName() == "attending" ) {
                m_attending = parseUidList( e );
            } else if ( e.tagName() == "unsure" ) {
                m_unsure = parseUidList( e );
            } else if ( e.tagName() == "declined" ) {
                m_declined = parseUidList( e );
            } else if ( e.tagName() == "not_replied" ) {
                m_notReplied = parseUidList( e );
            }
        }
        n = n.nextSibling();
    }
}


UidList parseUidList( const QDomElement & elem )
{
    UidList uids;
    QDomNode n = elem.firstChild();
    while (!n.isNull()) {
        QDomElement e = n.toElement(); // try to convert the node to an element.
        if (!e.isNull()) {
            if ( e.tagName() == "uid" ) {
                uids.append( e.text().toUInt() );
            }
        }
        n = n.nextSibling();
    }
    return uids;
}

KFacebook::GroupsGetJob::GroupsGetJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::GroupsGetJob::processResponse()
{
    kDebug() << endl;
    uint gid, nid, creator;
    QString name, description, type, subType, recentNews, office, website;
    KUrl smallPicUrl, medPicUrl, largePicUrl;
    QDateTime updateTime;
    Venue venue;

    QDomDocument doc = response();
    QDomElement docElem = doc.documentElement();
    QDomNode t = docElem.firstChild();
    while (!t.isNull()) {
        QDomElement f = t.toElement();

        if ( f.tagName() == "group" ) {
            QDomNode n = f.firstChild();
            while (!n.isNull()) {
                QDomElement e = n.toElement(); // try to convert the node to an element.
                if (!e.isNull()) {
                    QString tag = e.tagName();
                    if ( tag == "gid" ) {
                        gid = e.text().toUInt();
                    } else if ( tag == "name" ) {
                        name = e.text();
                    } else if ( tag == "nid" ) {
                        nid = e.text().toUInt();
                    } else if ( tag == "pic_small" ) {
                        smallPicUrl = KUrl( e.text() );
                    } else if ( tag == "pic" ) {
                        medPicUrl = KUrl( e.text() );
                    } else if ( tag == "pic_large" ) {
                        largePicUrl = KUrl( e.text() );
                    } else if ( tag == "recent_news" ) {
                        recentNews = e.text();
                    } else if ( tag == "description" ) {
                        description = e.text();
                    } else if ( tag == "event_type" ) {
                        type = e.text();
                    } else if ( tag == "event_subtype" ) {
                        subType = e.text();
                    } else if ( tag == "creator" ) {
                        creator = e.text().toUInt();
                    } else if ( tag == "update_time" ) {
                        updateTime.setTime_t( e.text().toUInt() );
                    } else if ( tag == "office" ) {
                        office = e.text();
                    } else if ( tag == "website" ) {
                        website = e.text();
                    } else if ( tag == "venue" ) {
                        venue = parseVenue( e );
                    }

                }
                n = n.nextSibling();
            }
            m_groups.append( KFacebook::Group( gid, name, description, nid, type, subType, recentNews, smallPicUrl, medPicUrl, largePicUrl, creator, updateTime, office, website, venue ) );
        }
        t = t.nextSibling();
    }
    foreach (KFacebook::Group grp, m_groups ) {
        grp.dump();
    }
}

KFacebook::GroupsGetMembersJob::GroupsGetMembersJob( const KUrl & url ) : KFacebook::Job( url )
{
}

void KFacebook::GroupsGetMembersJob::processResponse()
{
    QDomDocument doc = response();
    QDomElement docElem = doc.documentElement();
    QDomNode n = docElem.firstChild();
    while (!n.isNull()) {
        QDomElement e = n.toElement(); // try to convert the node to an element.
        if (!e.isNull()) {
            if ( e.tagName() == "members" ) {
                m_members = parseUidList( e );
            } else if ( e.tagName() == "admins" ) {
                m_admins = parseUidList( e );
            } else if ( e.tagName() == "officers" ) {
                m_officers = parseUidList( e );
            } else if ( e.tagName() == "not_replied" ) {
                m_notReplied = parseUidList( e );
            }
        }
        n = n.nextSibling();
    }
}

#if 0
KFacebook::PhotosAddTagJob::PhotosAddTagJob( const KUrl & url ) : KFacebook::Job( url )
{
}
#endif

#include "fbjobs.moc"

