/*  This file is part of the KDE project
    Copyright (C) 2007,2008 Will Stephenson <wstephenson@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library.  If not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.

*/

#ifndef FACEBOOKJOBS_H
#define FACEBOOKJOBS_H

#include <QDomDocument>
#include <QDateTime>
#include <QList>

#include <KUrl>
#include <kio/job.h>

#include <kfacebook/error.h>
#include <kfacebook/kfacebook_export.h>

#include "event.h"
#include "group.h"
#include "userinfo.h"

typedef QMap<QString,QString> ParamMap;
typedef QPair<uint, uint> UidPair;
typedef QList<UidPair> UidPairList;
typedef QList<uint> UidList;

namespace KFacebook
{

class KFACEBOOK_EXPORT Job : public KIO::SpecialJob
{
Q_OBJECT
public:
    Job( const KUrl & url );
    virtual ~Job();
    Error platformError() const;
    QDomDocument response() const;
protected:
    virtual void processResponse() {};
protected Q_SLOTS:
    virtual void slotFinished();
    virtual void slotData( const QByteArray & );
private:
    class JobPrivate;
    JobPrivate * d;
};

class KFACEBOOK_EXPORT InvalidJob : public Job
{
public:
    InvalidJob( const KUrl & url ) : Job( url ) {}
protected:
    void processResponse() {}
private:
};

class KFACEBOOK_EXPORT AuthCreateTokenJob : public Job
{
public:
    AuthCreateTokenJob( const KUrl & url );
    QString token() const { return m_token; }
protected:
    void processResponse();
private:
    QString m_token;
};

class KFACEBOOK_EXPORT AuthGetSessionJob : public Job
{
public:
    AuthGetSessionJob( const KUrl & url );
    QString sessionKey() { return m_sessionKey; }
    QString uid() { return m_uid; }
    QString secret() { return m_secret; }
    QDateTime expires() const { return m_expires; }
protected:
    void processResponse();
private:
    QString m_secret;
    QString m_sessionKey;
    QString m_uid;
    QDateTime m_expires;
};

class KFACEBOOK_EXPORT AuthLogoutJob : public Job
{
public:
    AuthLogoutJob( const KUrl & url );
protected:
    void processResponse();
};

class KFACEBOOK_EXPORT FriendsGetJob : public Job
{
public:
    FriendsGetJob( const KUrl & url );
    UidList friends() const { return m_friends; }
protected:
    void processResponse();
private:
    UidList m_friends;
};

class KFACEBOOK_EXPORT FriendsGetAppUsersJob : public Job
{
public:
    FriendsGetAppUsersJob( const KUrl & url );

protected:
    void processResponse();
    UidList appUsers() const { return m_appUsers; }
private:
    UidList m_appUsers;
};

class KFACEBOOK_EXPORT FriendsAreFriendsJob : public Job
{
public:
    FriendsAreFriendsJob( const KUrl & url );
    UidPairList friendsList() const { return m_friendsList; }
    UidPairList notFriendsList() const { return m_notFriendsList; }

protected:
    void processResponse();
    UidPairList m_friendsList;
    UidPairList m_notFriendsList;
};

class KFACEBOOK_EXPORT UsersGetInfoJob : public Job
{
public:
    UsersGetInfoJob( const KUrl & url );
    QList<UserInfo> userInfo() const { return m_userInfo; }
protected:
    void processResponse();
private:
    QList<UserInfo> m_userInfo;
};

class KFACEBOOK_EXPORT UsersGetLoggedInUserJob : public Job
{
public:
    UsersGetLoggedInUserJob( const KUrl & url );
    uint uid() const { return m_uid; }
protected:
    void processResponse();
private:
    uint m_uid;
};

class KFACEBOOK_EXPORT FeedPublishJob : public Job
{
public:
    FeedPublishJob( const KUrl & url );
    bool success() const { return m_success; }
protected:
    void processResponse();
private:
    bool m_success;
};

class KFACEBOOK_EXPORT FQLQueryJob : public Job
{
public:
    FQLQueryJob( const KUrl & url );
protected:
    void processResponse();
};

class KFACEBOOK_EXPORT NotificationsGetJob : public Job
{
public:
    NotificationsGetJob( const KUrl & url );
    uint unreadMessages() { return m_unreadMsgs; }
    uint lastMessage() { return m_lastMsg; }
    uint unreadPokes() { return m_unreadPokes; }
    uint lastPoke() { return m_lastPoke; }
    uint unreadShares() { return m_unreadShares; }
    uint lastShare() { return m_lastShare; }
    UidList friendRequests() { return m_friendRequests; }
    QList<uint> groupInvites() { return m_groupInvites; }
    QList<uint> eventInvites() { return m_eventInvites; }
protected:
    void processResponse();
    void parseNotificationsStanza( const QDomElement & elem, uint& unread, uint& last );
private:
    uint m_unreadMsgs, m_lastMsg, m_unreadPokes, m_lastPoke, m_unreadShares,
         m_lastShare;
    UidList m_friendRequests;
    QList<uint> m_groupInvites;
    QList<uint> m_eventInvites;
};

class KFACEBOOK_EXPORT NotificationsSendJob : public Job
{
public:
    NotificationsSendJob( const KUrl & url );
    KUrl confirmationUrl() const { return m_url; }
protected:
    void processResponse();
private:
    KUrl m_url;
};

class KFACEBOOK_EXPORT NotificationsSendRequestJob : public Job
{
public:
    NotificationsSendRequestJob( const KUrl & url );
    KUrl confirmationUrl() const { return m_url; }
protected:
    void processResponse();
private:
    KUrl m_url;
};

class KFACEBOOK_EXPORT ProfileSetFBMLJob : public Job
{
public:
    ProfileSetFBMLJob( const KUrl & url );
    bool success() const { return m_success; }
protected:
    void processResponse();
private:
    bool m_success;
};

class KFACEBOOK_EXPORT ProfileGetFBMLJob : public Job
{
public:
    ProfileGetFBMLJob( const KUrl & url );
    QString profileFBML() const { return m_fbml; }
protected:
    void processResponse();
    QString m_fbml;
};

class KFACEBOOK_EXPORT EventsGetJob : public Job
{
public:
    EventsGetJob( const KUrl & url );
    QList<Event> events() const { return m_events; }
protected:
    void processResponse();
    QList<Event> m_events;
};
class KFACEBOOK_EXPORT EventsGetMembersJob : public Job
{
public:
    EventsGetMembersJob( const KUrl & url );
    UidList attending() const { return m_attending; }
    UidList unsure() const { return m_unsure; }
    UidList declined() const { return m_declined; }
    UidList notReplied() const { return m_notReplied; }
protected:
    void processResponse();
private:
    UidList m_attending;
    UidList m_unsure;
    UidList m_declined;
    UidList m_notReplied;
};

class KFACEBOOK_EXPORT GroupsGetJob : public Job
{
public:
    GroupsGetJob( const KUrl & url );
    QList<Group> groups() const { return m_groups; }
protected:
    void processResponse();
    QList<Group> m_groups;
};

class KFACEBOOK_EXPORT GroupsGetMembersJob : public Job
{
public:
    GroupsGetMembersJob( const KUrl & url );
    UidList members() const { return m_members; }
    UidList admins() const { return m_admins; }
    UidList officers() const { return m_officers; }
    UidList notReplied() const { return m_notReplied; }
protected:
    void processResponse();
private:
    UidList m_members;
    UidList m_admins;
    UidList m_officers;
    UidList m_notReplied;
};
#if 0
class KFACEBOOK_EXPORT PhotosAddTagJob : public Job
{
public:
    PhotosAddTagJob( const KUrl & url );
protected:
    void processResponse();
};
class KFACEBOOK_EXPORT PhotosCreateAlbumJob : public Job
{
public:
    ( const KUrl & url );
protected:
    void processResponse();
};
class KFACEBOOK_EXPORT PhotosGetJob : public Job
{
public:
    PhotosGetJob( const KUrl & url );
protected:
    void processResponse();
};

class KFACEBOOK_EXPORT PhotosGetAlbumsJob : public Job
{
public:
    PhotosGetAlbumsJob( const KUrl & url );
protected:
    void processResponse();
};

class KFACEBOOK_EXPORT PhotosGetTagsJob : public Job
{
public:
    PhotosGetTagsJob( const KUrl & url );
protected:
    void processResponse();
};

class KFACEBOOK_EXPORT PhotosUploadJob : public Job
{
public:
    PhotosUploadJob( const KUrl & url );
protected:
    void processResponse();
};
#endif

} // namespace KFacebook

#endif // FACEBOOKJOBS_H
