/*  This file is part of the KDE project
    Copyright (C) 2007,2008 Will Stephenson <wstephenson@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library.  If not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.

*/

#include <QDateTime>
#include <QList>
#include <QTimer>

#include <KConfig>
#include <KDebug>
#include <KLocale>
#include <ktoolinvocation.h>
#include <kconfiggroup.h>
#include <kurl.h>


#include "fbjobs.h"
#include "method.h"
#include "signature.h"
#include "userinfoxmlhelper.h"
#include "version.h"

#include "session.h"

namespace KFacebook
{
class Session::Private
{
public:
    Private( const QString & theApiKey, const QString & theSecretKey ) : apiKey( theApiKey ), secretKey( theSecretKey ), callId( 1 )
    { }
    uint nextCallId() { return callId++; }
    Session::State state;
    QDateTime expires;
    QString token;
    QString uid;
    const QString apiKey;
    const QString secretKey;
    QString sessionKey;
    QString sessionSecret;
    Job * job;
private:
    uint callId;
};
} // namespace KFacebook

typedef QPair<QString,QString> ParamPair;

KFacebook::Session::Session( QObject * parent, const QString & apiKey, const QString & secretKey ) : QObject( parent ), d( new KFacebook::Session::Private( apiKey, secretKey ) )
{
    KConfig config( QString::fromLatin1( "libkfacebookrc" ) );
    KConfigGroup group = config.group("Session");
    QString sessionKey = group.readEntry( "Key" );

    QVariant defaultExp = QDateTime::currentDateTime();
    QVariant expiry = group.readEntry( "Expires", defaultExp );
    d->uid  = group.readEntry( "Uid" );
    QDateTime sessionExpires = expiry.toDateTime();
    QString sessionSecret = group.readEntry( "Secret" );
    if ( sessionExpires < QDateTime::currentDateTime() || sessionKey.isNull() || sessionSecret.isEmpty() ) {
        kDebug() << "Found no session or expired session" << endl;
        d->state = NotAuthenticated;
    } else {
        kDebug() << "Found valid session" << endl;
        d->sessionKey = sessionKey;
        d->sessionSecret = sessionSecret;
        d->expires = sessionExpires;
        d->state = Authenticated;
    }
    nextStep();
}

KFacebook::Session::~Session()
{
    delete d;
}

KFacebook::Session::State KFacebook::Session::state() const
{
    return d->state;
}

QString KFacebook::Session::apiKey() const
{
    return d->apiKey;
}

QString KFacebook::Session::secretKey() const
{
    return d->secretKey;
}

QString KFacebook::Session::sessionSecret() const
{
    return d->sessionSecret;
}

QString KFacebook::Session::uid() const
{
    return d->uid;
}

// helper methods
void KFacebook::Session::addGenericParams( ParamMap & params )
{
    params.insert( KFacebook::Method::PARAM_VERSION, KFACEBOOK_FB_API_VERSION );
    params.insert( KFacebook::Method::PARAM_API_KEY, d->apiKey );
}

void KFacebook::Session::addSessionParams( ParamMap & params )
{
    addGenericParams( params );
    params.insert( KFacebook::Method::PARAM_SESSION_KEY, d->sessionKey );
    if ( params.contains( KFacebook::Method::PARAM_METHOD )
            && ( params[KFacebook::Method::PARAM_METHOD] != KFacebook::Method::AUTH_CREATETOKEN
                && params[KFacebook::Method::PARAM_METHOD] != KFacebook::Method::AUTH_GETSESSION
               )
       ) {
        params.insert( KFacebook::Method::PARAM_CALL_ID, QString::number( (float)d->nextCallId(), 'f', 1 ) );
    }
}

KUrl KFacebook::Session::paramsToKUrl( const ParamMap & params )
{
    KUrl url( "http://api.facebook.com/restserver.php" );

    // use https for getSession
    if ( params.contains( KFacebook::Method::PARAM_METHOD ) && params[KFacebook::Method::PARAM_METHOD] == KFacebook::Method::AUTH_GETSESSION ) {
        url.setProtocol( "https" );
    }

    // first generate the signature of the un-percent-encoded params
    KFacebook::Signature signature;
    QMapIterator<QString,QString> i(params);
    QStringList stringParams;
    while (i.hasNext()) {
        i.next();
        stringParams.append( QString("%1=%2").arg( i.key() ).arg( i.value() ));
    }
    // use app's secret key for signing session setup calls, session key after that
    QString signingKey;
    if ( params.contains( KFacebook::Method::PARAM_METHOD ) && ( params[KFacebook::Method::PARAM_METHOD] == KFacebook::Method::AUTH_CREATETOKEN || params[KFacebook::Method::PARAM_METHOD] == KFacebook::Method::AUTH_GETSESSION ) ) {
        kDebug() << "Using app secret key" << endl;
        signingKey = d->secretKey;
    } else {
        kDebug() << "Using session secret: " << d->sessionSecret << endl;
        signingKey = d->sessionSecret;
    }
    QString sig = QString( "sig=%1" ).arg( QString( signature.generate( stringParams, signingKey ) ) );
    // now percent-encode the parameters.
    stringParams.clear();
    i.toFront();
    while (i.hasNext()) {
        i.next();
        stringParams.append( QString("%1=").arg( i.key() ).append( QUrl::toPercentEncoding( i.value() ) ));
    }
    stringParams.prepend( sig );
    QString joined = stringParams.join("&" );

    url.setQuery( joined );
    kDebug() << "FINISHED URL: " << url << endl;
    return url;
}


// state methods
void KFacebook::Session::getToken()
{
    d->job = authCreateTokenJob( );
    connect ( d->job, SIGNAL( result( KJob * ) ), SLOT( gotTokenResponse() ) );
}

void KFacebook::Session::nextStep()
{
    QList<uint> groups; 
    switch ( d->state ) {
        case NotAuthenticated:
            getToken();
            break;
        case GotToken:
            showBrowser();
            break;
        case WaitingForSession:
            getSession();
            break;
        case Authenticated:
            emit ready();
            //getUsersGetInfo();
            //feedPublishStoryToUserJob( QString("pulled a lever" ), QString("What does this button do.."), "", "", "", "", "", "", "", "", 1 );
            //usersGetLoggedInUserJob();
            //getFriendsAreFriends();
            // seems that setFBML is only useful if the app has been added to the user's account
            //profileSetFBMLJob("<span id=\"cdc03\" style=\"background: yellow\">this can appear and disappear</span><input type=\"submit\" clicktohide=\"cdc03\" value=\"hide\"/><input type=\"submit\" clicktoshow=\"cdc03\" value=\"show\"/>",693961002);
            //eventsGetMembersJob(2325324685);
            //groups.append(2649735358);
            //groupsGetJob(0, groups);
            //groupsGetMembersJob(2649735358);
            //eventsGetJob(693961002);
            //notificationsGetJob();
            //authLogoutJob();
            break;
        default:
            break;
    }
}

void KFacebook::Session::gotTokenResponse()
{
    kDebug() << endl;
    if ( d->job ) {
        if ( d->job->error() ) {
            kDebug() << "error getting Token: " << d->job->errorString() << endl;
        } else if ( d->job->platformError().isValid() ) {
            KFacebook::Error err = d->job->platformError();
            kDebug() << "PLATFORM ERROR: " << err.code() << ", '" << err.toString() << endl;
            d->state = Error;
        } else {
            KFacebook::AuthCreateTokenJob * job = static_cast<KFacebook::AuthCreateTokenJob*>(d->job);
            d->token = job->token();
        }
        d->job = 0;
    }
    if ( !d->token.isEmpty() )
    {
        d->state = GotToken;
        nextStep();
    } else {
        d->state = Error;
    }
}

void KFacebook::Session::showBrowser()
{
    QString loginUrl = "http://www.facebook.com/login.php?api_key=%1&v=%2&auth_token=%3";
    loginUrl = loginUrl.arg( d->apiKey ).arg( KFACEBOOK_FB_API_VERSION ).arg( d->token );
    kDebug() << "LOGIN URL: " << loginUrl << endl;

    KToolInvocation::invokeBrowser( loginUrl );
}

void KFacebook::Session::browserLoggedIn()
{
    kDebug() << endl;
    if ( d->state == GotToken )
    {
        kDebug() << "  changing from GotToken -> WaitingForSession, requesting session" << endl;
        d->state = WaitingForSession;
        nextStep();
    }
}

void KFacebook::Session::getSession()
{
    Q_ASSERT( !d->token.isEmpty() );
    d->job = authGetSessionJob();
    connect ( d->job, SIGNAL( result( KJob * ) ), SLOT( gotSessionResponse() ) );
}

void KFacebook::Session::gotSessionResponse()
{
    kDebug() << endl;
    if ( d->job ) {
        if ( d->job->error() ) {
            kDebug() << "error getting session: " << d->job->errorString() << endl;
        } else if ( d->job->platformError().isValid() ) {
            KFacebook::Error err = d->job->platformError();
            kDebug() << "PLATFORM ERROR: " << err.code() << ", '" << err.toString() << endl;
            d->state = Error;
        } else {
            KFacebook::AuthGetSessionJob * job = static_cast<KFacebook::AuthGetSessionJob*>(d->job);
            d->sessionKey = job->sessionKey();
            d->expires = job->expires();
            d->uid = job->uid();
            d->sessionSecret = job->secret();
            // persist session info
            KConfig config( QString::fromLatin1( "libkfacebookrc" ) );
            KConfigGroup group = config.group("Session");
            group.writeEntry( "Key", d->sessionKey );
            group.writeEntry( "Expires", d->expires );
            group.writeEntry( "Secret", d->sessionSecret );
            group.writeEntry( "Uid", d->uid );

            kDebug() << "SESSION SECRET: " << d->sessionSecret << " SESSION KEY: " << d->sessionKey << " UID: " << d->uid << " EXPIRES: " << d->expires << endl;
            if ( d->sessionKey.isEmpty() || d->sessionSecret.isEmpty() || d->uid.isEmpty() || !d->expires.isValid() ) {
                kDebug() << "getSession failed, check auth.getSession response" << endl;
                d->state = Error;
            } else {
                d->state = Authenticated;
            }
        }
        d->job = 0;
    }
    nextStep();
}

#if 0
void KFacebook::Session::getFriends()
{
    Q_ASSERT( !d->sessionKey.isEmpty() );
    d->job = KFacebook::Session::friendsGetJob();
    connect ( d->job, SIGNAL( result( KJob * ) ), SLOT( gotFriendsGetResponse() ) );
}

void KFacebook::Session::getFriendsAreFriends()
{
    Q_ASSERT( !d->sessionKey.isEmpty() );
    QList<int> uids1;
    QList<int> uids2;
    uids1 << 515703001 << 754315455;
    uids2 << 675197176 << 515703001;

    d->job = KFacebook::Session::friendsAreFriendsJob( uids1, uids2 );
    connect ( d->job, SIGNAL( result( KJob * ) ), SLOT( gotFriendsAreFriendsResponse() ) );
    connect ( this, SIGNAL( gotFriendsAreFriends( const FriendPairList &, const FriendPairList & ) ),
            this, SLOT( receiveFriendsAreFriends( const FriendPairList &, const FriendPairList & ) ) );
}

void KFacebook::Session::getUsersGetInfo()
{
    Q_ASSERT( !d->sessionKey.isEmpty() );
    QList<int> uids1;
    uids1 /*<< 526650328*/ << 693961002;

    QStringList fields;
    fields << KFacebook::Method::FIELD_NAME << KFacebook::Method::FIELD_ABOUT_ME
        << KFacebook::Method::FIELD_BIRTHDAY << KFacebook::Method::FIELD_BOOKS
        << KFacebook::Method::FIELD_FIRST_NAME << KFacebook::Method::FIELD_LAST_NAME
        << KFacebook::Method::FIELD_INTERESTS << KFacebook::Method::FIELD_LOOKING_FOR
        << KFacebook::Method::FIELD_INTERESTED_IN << KFacebook::Method::FIELD_MOVIES
        << KFacebook::Method::FIELD_MUSIC << KFacebook::Method::FIELD_NOTES_COUNT
        << KFacebook::Method::FIELD_PICTURE_SMALL << KFacebook::Method::FIELD_PICTURE_MEDIUM
        << KFacebook::Method::FIELD_PICTURE_LARGE << KFacebook::Method::FIELD_PICTURE_SQUARE
        << KFacebook::Method::FIELD_POLITICAL << KFacebook::Method::FIELD_PROFILE_UPDATE_TIME
        << KFacebook::Method::FIELD_QUOTES << KFacebook::Method::FIELD_RELATIONSHIP_STATUS
        << KFacebook::Method::FIELD_RELIGION <<     KFacebook::Method::FIELD_SEX
        << KFacebook::Method::FIELD_SIGNIFICANT_OTHER << KFacebook::Method::FIELD_STATUS
        << KFacebook::Method::FIELD_TIMEZONE << KFacebook::Method::FIELD_TV
        << KFacebook::Method::FIELD_WALL_COUNT << KFacebook::Method::FIELD_AFFILIATIONS
        << KFacebook::Method::FIELD_CURRENT_LOCATION << KFacebook::Method::FIELD_HOMETOWN_LOCATION
        << KFacebook::Method::FIELD_EDUCATION_HISTORY << KFacebook::Method::FIELD_HIGHSCHOOL_INFO
        << KFacebook::Method::FIELD_WORK_HISTORY;

    d->job = KFacebook::Session::usersGetInfoJob( uids1, fields );
    connect ( d->job, SIGNAL( result( KJob * ) ), SLOT( gotUsersGetInfoResponse() ) );
}

void KFacebook::Session::gotFriendsGetResponse()
{
    kDebug() << endl;
    if ( d->job ) {
        if ( d->job->error() ) {
            kDebug() << "error getting friends: " << d->job->errorString() << endl;
        } else if ( d->job->platformError().isValid() ) {
            KFacebook::Error err = d->job->platformError();
            kDebug() << "PLATFORM ERROR: " << err.code() << ", '" << err.toString() << endl;
            d->state = Error;
        } else {
            KFacebook::FriendsGetJob* job = static_cast<KFacebook::FriendsGetJob*>(d->job);
            d->job = 0;
            emit gotFriends( job->friends() );
        }
    }
}

void KFacebook::Session::gotFriendsAreFriendsResponse()
{
    kDebug() << endl;
    if ( d->job ) {
        if ( d->job->error() ) {
            kDebug() << "error getting friends: " << d->job->errorString() << endl;
        } else if ( d->job->platformError().isValid() ) {
            KFacebook::Error err = d->job->platformError();
            kDebug() << "PLATFORM ERROR: " << err.code() << ", '" << err.toString() << endl;
            d->state = Error;
        } else {
            KFacebook::FriendsAreFriendsJob * job = static_cast<KFacebook::FriendsAreFriendsJob*>(d->job);
            d->job = 0;
            UidPairList friendsList, notFriendsList;
            friendsList = job->friendsList();
            notFriendsList = job->notFriendsList();
            emit gotFriendsAreFriends( friendsList, notFriendsList );
        }
        d->job = 0;
    }
}

void KFacebook::Session::gotUsersGetInfoResponse()
{
    kDebug() << endl;
    if ( d->job ) {
        if ( d->job->error() ) {
            kDebug() << "error getting friends: " << d->job->errorString() << endl;
        } else if ( d->job->platformError().isValid() ) {
            KFacebook::Error err = d->job->platformError();
            kDebug() << "PLATFORM ERROR: " << err.code() << ", '" << err.toString() << endl;
            d->state = Error;
        } else {
            KFacebook::UsersGetInfoJob * job = static_cast<KFacebook::UsersGetInfoJob*>(d->job);

            emit gotUsersGetInfo( job->userInfo() );
        }
    }
}
#endif

// job factory methods

KFacebook::Job * KFacebook::Session::authCreateTokenJob(/* const ParamMap & params */)
{
    ParamMap myParams;
    myParams.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::AUTH_CREATETOKEN );
    addGenericParams( myParams );
    return new KFacebook::AuthCreateTokenJob( paramsToKUrl( myParams ) );
}

KFacebook::Job * KFacebook::Session::authGetSessionJob()
{
    if( d->token.isEmpty() )
        return 0;
    ParamMap myParams;
    myParams.insert( KFacebook::Method::PARAM_AUTH_TOKEN, d->token );
    myParams.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::AUTH_GETSESSION );
    addGenericParams( myParams );
    return new KFacebook::AuthGetSessionJob( paramsToKUrl( myParams ) );
}

KFacebook::Job * KFacebook::Session::authLogoutJob()
{
    kDebug() << endl;
    KUrl logoutUrl( "http://www.facebook.com/logout.php?confirm=1" );
    return new KFacebook::AuthLogoutJob( logoutUrl );
}

KFacebook::Job * KFacebook::Session::friendsGetJob()
{
    if( d->sessionKey.isEmpty() )
        return 0;
    ParamMap myParams;
    myParams.insert( KFacebook::Method::PARAM_AUTH_TOKEN, d->token );
    myParams.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::FRIENDS_GET );
    addSessionParams( myParams );
    return new KFacebook::FriendsGetJob( paramsToKUrl( myParams ) );
}

KFacebook::Job * KFacebook::Session::friendsAreFriendsJob( const QList<uint> & uids1, const QList<uint> & uids2 )
{
    if ( d->sessionKey.isEmpty() || ( uids1.count() == uids2.count() ) )
        return 0;
    QStringList firstUids, secondUids;
    for ( int i = 0; i < uids1.count(); ++i ) {
        firstUids.append( QString::number( uids1[i] ) );
        secondUids.append( QString::number( uids2[i] ) );
    }

    ParamMap myParams;
    myParams.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::FRIENDS_AREFRIENDS );
    myParams.insert( KFacebook::Method::PARAM_FRIENDS_UIDS1, firstUids.join( "," ) );
    myParams.insert( KFacebook::Method::PARAM_FRIENDS_UIDS2, secondUids.join( "," ) );
    addSessionParams( myParams );
    return new KFacebook::FriendsAreFriendsJob( paramsToKUrl( myParams ) );
}

KFacebook::Job * KFacebook::Session::usersGetInfoJob( const QList<uint> & uids, const QStringList & fields )
{
    if ( d->sessionKey.isEmpty() || uids.isEmpty() || fields.isEmpty() )
        return 0;
    ParamMap params;
    QStringList uidStrings;
    for ( int i = 0; i < uids.count(); ++i ) {
        uidStrings.append( QString::number( uids[i] ) );
    }
    params.insert( KFacebook::Method::PARAM_UIDS, uidStrings.join( "," ) );
    params.insert( KFacebook::Method::PARAM_FIELDS, fields.join( "," ) );
    params.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::USERS_GETINFO );
    addSessionParams( params );
    return new KFacebook::UsersGetInfoJob( paramsToKUrl( params ) );
}

KFacebook::Job * KFacebook::Session::usersGetLoggedInUserJob()
{
    if ( d->sessionKey.isEmpty() )
        return 0;
    ParamMap params;
    params.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::USERS_GETLOGGEDINUSER );
    addSessionParams( params );
    return new KFacebook::UsersGetLoggedInUserJob( paramsToKUrl( params ) );
}

KFacebook::Job * KFacebook::Session::feedPublishStoryToUserJob( const QString & title, const QString & body,
        const QString & image1 = "", const QString & image1Link = "",
        const QString & image2 = "", const QString & image2Link = "",
        const QString & image3 = "", const QString & image3Link = "",
        const QString & image4 = "", const QString & image4Link = "", int priority = 0)
{
    if ( d->sessionKey.isEmpty() )
        return 0;
    ParamMap params;
    params.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::FEED_PUBLISHSTORYTOUSER );
    return feedPublishJob( params, title, body, image1, image1Link, image2, image2Link, image3, image3Link,
            image4, image4Link, priority );
}

KFacebook::Job * KFacebook::Session::feedPublishActionOfUserJob( const QString & title, const QString & body,
        const QString & image1 = "", const QString & image1Link = "",
        const QString & image2 = "", const QString & image2Link = "",
        const QString & image3 = "", const QString & image3Link = "",
        const QString & image4 = "", const QString & image4Link = "", int priority = 0)
{
    if ( d->sessionKey.isEmpty() )
        return 0;
    ParamMap params;
    params.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::FEED_PUBLISHACTIONOFUSER );
    return feedPublishJob( params, title, body, image1, image1Link, image2, image2Link, image3, image3Link,
            image4, image4Link, priority );
}

KFacebook::Job * KFacebook::Session::feedPublishJob( ParamMap & params, const QString & title, const QString & body,
            const QString & image1, const QString & image1Link,
            const QString & image2, const QString & image2Link,
            const QString & image3, const QString & image3Link,
            const QString & image4, const QString & image4Link, int priority )
{
    if ( d->sessionKey.isEmpty() )
        return 0;
    params.insert( KFacebook::Method::PARAM_FEED_TITLE, title );
    params.insert( KFacebook::Method::PARAM_FEED_BODY, body );
    if ( !image1.isEmpty() ) {
        params.insert( KFacebook::Method::PARAM_FEED_IMAGE1, image1 );
        if ( !image1Link.isEmpty() ) {
            params.insert( KFacebook::Method::PARAM_FEED_IMAGE1LINK, image1Link );
        }
    }
    if ( !image2.isEmpty() ) {
        params.insert( KFacebook::Method::PARAM_FEED_IMAGE2, image2 );
        if ( !image2Link.isEmpty() ) {
            params.insert( KFacebook::Method::PARAM_FEED_IMAGE2LINK, image2Link );
        }
    }
    if ( !image3.isEmpty() ) {
        params.insert( KFacebook::Method::PARAM_FEED_IMAGE3, image3 );
        if ( !image3Link.isEmpty() ) {
            params.insert( KFacebook::Method::PARAM_FEED_IMAGE3LINK, image3Link );
        }
    }
    if ( !image4.isEmpty() ) {
        params.insert( KFacebook::Method::PARAM_FEED_IMAGE4, image4 );
        if ( !image4Link.isEmpty() ) {
            params.insert( KFacebook::Method::PARAM_FEED_IMAGE4LINK, image4Link );
        }
    }
    if ( priority != 0 ) {
        params.insert( KFacebook::Method::PARAM_FEED_PRIORITY, QString::number( priority ) );
    }
    addSessionParams( params );
    return new KFacebook::FeedPublishJob( paramsToKUrl( params ) );
}

KFacebook::Job * KFacebook::Session::fqlQueryJob( const QString & query )
{
    if ( d->sessionKey.isEmpty() || query.isEmpty() )
        return 0;
    ParamMap params;
    params.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::FQL_QUERY );
    params.insert( KFacebook::Method::PARAM_QUERY, query );
    addSessionParams( params );
    return new KFacebook::FQLQueryJob( paramsToKUrl( params ) );
}

KFacebook::Job * KFacebook::Session::friendsGetAppUsersJob()
{
    if ( d->sessionKey.isEmpty() )
        return 0;
    ParamMap params;
    params.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::FRIENDS_GETAPPUSERS );
    addSessionParams( params );
    return new KFacebook::FriendsGetAppUsersJob( paramsToKUrl( params ) );
}

KFacebook::Job * KFacebook::Session::notificationsGetJob()
{
    if ( d->sessionKey.isEmpty() )
        return 0;
    ParamMap params;
    params.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::NOTIFICATIONS_GET );
    addSessionParams( params );
    return new KFacebook::NotificationsGetJob( paramsToKUrl( params ) );
}

KFacebook::Job * KFacebook::Session::notificationsSendJob( const QList<uint> & recipients, const QString & content, bool noEmail )
{
    if ( d->sessionKey.isEmpty() || recipients.isEmpty() )
        return 0;
    ParamMap params;
    params.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::NOTIFICATIONS_SEND );
    QStringList recipStrings;
    foreach ( uint uid, recipients ) {
        recipStrings.append( QString::number( uid ) );
    }
    params.insert( KFacebook::Method::PARAM_NOTIFICATIONS_TO_IDS, recipStrings.join(",") );
    params.insert( KFacebook::Method::PARAM_NOTIFICATIONS_MARKUP, content );
    params.insert( KFacebook::Method::PARAM_NOTIFICATIONS_NO_EMAIL, (noEmail ? "true" : "false") );
    addSessionParams( params );
    return new KFacebook::NotificationsSendJob( paramsToKUrl( params ) );
}

KFacebook::Job * KFacebook::Session::notificationsSendRequestJob( const QList<uint> & recipients, const QString & noun, const QString & content, const QString & image, RequestType type )
{
    if ( d->sessionKey.isEmpty() || recipients.isEmpty() )
        return 0;
    ParamMap params;
    QStringList recipStrings;
    foreach ( uint uid, recipients ) {
        recipStrings.append( QString::number( uid ) );
    }
    params.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::NOTIFICATIONS_SENDREQUEST );
    params.insert( KFacebook::Method::PARAM_NOTIFICATIONS_TO_IDS, recipStrings.join(",") );
    params.insert( KFacebook::Method::PARAM_NOTIFICATIONS_TYPE, noun );
    params.insert( KFacebook::Method::PARAM_NOTIFICATIONS_MARKUP, content );
    params.insert( KFacebook::Method::PARAM_NOTIFICATIONS_IMAGE, image );
    params.insert( KFacebook::Method::PARAM_NOTIFICATIONS_INVITE, ( type == KFacebook::Session::Invitation ? "invitation" : "request" ) );
    addSessionParams( params );
    return new KFacebook::NotificationsSendRequestJob( paramsToKUrl( params ) );
}

KFacebook::Job * KFacebook::Session::profileSetFBMLJob( const QString & markup, uint uid )
{
    if ( d->sessionKey.isEmpty() || markup.isEmpty() )
        return 0;
    ParamMap params;
    params.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::PROFILE_SETFBML );
    params.insert( KFacebook::Method::PARAM_NOTIFICATIONS_MARKUP, markup );
    params.insert( KFacebook::Method::PARAM_UID, QString::number( uid ) );
    addSessionParams( params );
    return new KFacebook::ProfileSetFBMLJob( paramsToKUrl( params ) );
}

KFacebook::Job * KFacebook::Session::profileGetFBMLJob( uint uid )
{
    if ( d->sessionKey.isEmpty() )
        return 0;
    ParamMap params;
    params.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::PROFILE_GETFBML );
    params.insert( KFacebook::Method::PARAM_UID, QString::number( uid ) );
    addSessionParams( params );
    return new KFacebook::ProfileGetFBMLJob( paramsToKUrl( params ) );
}

KFacebook::Job * KFacebook::Session::eventsGetJob( uint uid, const QList<uint> & eids, const QDateTime & startTime, const QDateTime & endTime, RSVPStatus rsvpStatus )
{
    if ( d->sessionKey.isEmpty() )
        return 0;
    ParamMap params;
    params.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::EVENTS_GET );
    if ( uid ) {
        params.insert( KFacebook::Method::PARAM_UID, QString::number( uid ) );
    }
    if ( eids.count() ) {
        QStringList eidList;
        foreach ( uint uid, eids ) {
            eidList.append( QString::number( uid ) );
        }
        params.insert( KFacebook::Method::PARAM_EVENTS_EIDS, eidList.join(",") );
    }
    if ( startTime.isValid() ) {
        params.insert( KFacebook::Method::PARAM_EVENTS_START_TIME, QString::number( startTime.toTime_t() ) );
    }
    if ( endTime.isValid() ) {
        params.insert( KFacebook::Method::PARAM_EVENTS_END_TIME, QString::number( endTime.toTime_t() ) );
    }
    if ( rsvpStatus != KFacebook::Session::RSVPNull ) {
        QString rsvpString;
        switch ( rsvpStatus ) {
            case KFacebook::Session::Attending:
                rsvpString = "attending";
                break;
            case KFacebook::Session::Unsure:
                rsvpString = "unsure";
                break;
            case KFacebook::Session::Declined:
                rsvpString = "declined";
                break;
            case KFacebook::Session::NotReplied:
                rsvpString = "not_replied";
                break;
            default:
                break;
        }
        params.insert( KFacebook::Method::PARAM_EVENTS_RSVP_STATUS, rsvpString );
    }
    addSessionParams( params );
    return new KFacebook::EventsGetJob( paramsToKUrl( params ) );
}

KFacebook::Job * KFacebook::Session::eventsGetMembersJob( uint eid )
{
    if ( d->sessionKey.isEmpty() )
        return 0;
    ParamMap params;
    params.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::EVENTS_GETMEMBERS );
    params.insert( KFacebook::Method::PARAM_EVENTS_EID, QString::number( eid ) );
/*2325324685*/
    addSessionParams( params );
    return new KFacebook::EventsGetMembersJob( paramsToKUrl( params ) );
}

KFacebook::Job * KFacebook::Session::groupsGetJob( uint uid, const QList<uint> & gids )
{
    if ( d->sessionKey.isEmpty() )
        return 0;
    ParamMap params;
    params.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::GROUPS_GET );
    if ( uid )
        params.insert( KFacebook::Method::PARAM_UID, QString::number( uid ) );
    if ( gids.count() ) {
        QStringList gidList;
        foreach ( uint uid, gids ) {
            gidList.append( QString::number( uid ) );
        }
        params.insert( KFacebook::Method::PARAM_GROUPS_GIDS, gidList.join(",") );
    }

    addSessionParams( params );
    return new KFacebook::GroupsGetJob( paramsToKUrl( params ) );
}

KFacebook::Job * KFacebook::Session::groupsGetMembersJob( uint gid )
{
    if ( d->sessionKey.isEmpty() )
        return 0;
    ParamMap params;
    params.insert( KFacebook::Method::PARAM_METHOD, KFacebook::Method::GROUPS_GETMEMBERS );
    params.insert( KFacebook::Method::PARAM_GROUPS_GID, QString::number( gid ) );
    addSessionParams( params );
    return new KFacebook::GroupsGetMembersJob( paramsToKUrl( params ) );

}

#if 0
// test slot to see if we are parsing the xml right
void KFacebook::Session::receiveFriendsAreFriends( const UidPairList & are, const UidPairList & arent )
{
    kDebug() << "Are friends: " << endl;;
    foreach ( UidPair pair, are ) {
        kDebug() << pair.first << "," << pair.second << endl;
    }
    kDebug() << "Are not friends: " << endl;;
    foreach ( UidPair pair, arent ) {
        kDebug() << pair.first << "," << pair.second << endl;
    }
}
#endif 
#include "session.moc"
