/*  This file is part of the KDE project
    Copyright (C) 2007,2008 Will Stephenson <wstephenson@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library.  If not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.

*/

#ifndef KFACEBOOK_SESSION_H
#define KFACEBOOK_SESSION_H

#include <QObject>

#include <kfacebook/kfacebook_export.h>
#include <kfacebook/fbjobs.h>

#include "userinfo.h"

typedef QPair<uint, uint> UidPair;
typedef QList<UidPair> UidPairList;

namespace KFacebook
{
class KFACEBOOK_EXPORT Session : public QObject
{
Q_OBJECT
public:
    enum State { NotAuthenticated, GotToken, WaitingForLogin, WaitingForSession, Authenticated, Error };
    enum RequestType { Invitation, Request };
    enum RSVPStatus { RSVPNull, Attending, Unsure, Declined, NotReplied };

    Session( QObject * parent, const QString & apiKey, const QString & secretKey );
    virtual ~Session();
    State state() const;
    QString apiKey() const;
    QString secretKey() const;
    QString sessionSecret() const;
    QString uid() const;

    //TODO: move these to AuthJobFactory, FriendsJobFactory etc
    Job * authCreateTokenJob();
    Job * authGetSessionJob();
    Job * authLogoutJob();
    Job * friendsGetJob();
    Job * friendsAreFriendsJob( const QList<uint> & uids1, const QList<uint> & uids2 );
    Job * usersGetInfoJob( const QList<uint> & uids, const QStringList & fields );
    Job * usersGetLoggedInUserJob();
    // TODO: Fix permissions error returned by this job
    Job * feedPublishStoryToUserJob( const QString & title, const QString & body,
            const QString & image1, const QString & image1Link,
            const QString & image2, const QString & image2Link,
            const QString & image3, const QString & image3Link,
            const QString & image4, const QString & image4Link, int priority );
    Job * feedPublishActionOfUserJob( const QString & title, const QString & body,
            const QString & image1, const QString & image1Link,
            const QString & image2, const QString & image2Link,
            const QString & image3, const QString & image3Link,
            const QString & image4, const QString & image4Link, int priority );
    Job * fqlQueryJob( const QString & query );
    Job * friendsGetAppUsersJob();
    Job * notificationsGetJob();
    Job * notificationsSendJob( const QList<uint> & recipients, const QString & content, bool noEmail );
    Job * notificationsSendRequestJob( const QList<uint> & recipients, const QString & noun, const QString & content, const QString & image, RequestType type );
    Job * profileSetFBMLJob( const QString & markup, uint uid = 0 );
    Job * profileGetFBMLJob( uint uid = 0 );
    
    //Job * updateDecodeIDsJob( const QList<int> & uids );
    Job * eventsGetJob( uint uid = 0, const QList<uint> & eids = QList<uint>(), const QDateTime & startTime = QDateTime(), const QDateTime & endTime = QDateTime(), RSVPStatus = RSVPNull );
    Job * eventsGetMembersJob( uint eid );
    Job * groupsGetJob( uint uid = 0, const QList<uint> & gids = QList<uint>() );
    Job * groupsGetMembersJob( uint gid );
    // call implemented as far as HERE
    // returned info processed as far as HERE
    Job * photosAddTagJob( uint pid, uint tagUid, const QString & text, double x, double y, const QString & jsonTags = "" );
    Job * photosCreateAlbumJob( const QString & name, const QString & location, const QString & description );
    Job * photosGetJob( uint subjectUid, uint albumId, const QList<uint> & pids = QList<uint>() );
    Job * photosGetAlbumsJob( uint uid, const QList<uint> & aids = QList<uint>() );
    Job * photosGetTagsJob( const QList<uint> & pids );
    Job * photosUploadJob( uint aid, const QString & caption, const QByteArray & mimeData );

signals:
//X     void gotFriends( QList<uint> );
//X     void gotFriendsAreFriends( const UidPairList &, const UidPairList & );
//X     void gotUsersGetInfo( const QList<KFacebook::UserInfo> & );
    void ready();
public Q_SLOTS:
    void browserLoggedIn();
protected Q_SLOTS:
    void gotTokenResponse();
    void gotSessionResponse();
//X     void gotFriendsGetResponse();
//X     void gotFriendsAreFriendsResponse();
//X     void gotUsersGetInfoResponse();

//X     void receiveFriendsAreFriends( const UidPairList & are, const UidPairList & arent );
private:
    // advance the state machine
    void nextStep();
    // state machine steps
    void getToken();
    void showBrowser();
    void getSession();
    // test actually doing something - these methods will eventually be public session api and have
    // parameters
//X     void getFriends();
//X     void getFriendsAreFriends();
//X     void getUsersGetInfo();
    // setup job parameters
    KUrl paramsToKUrl( const ParamMap & params );
    void addGenericParams( ParamMap & params );
    void addSessionParams( ParamMap & params );
    // init feed job parameters
    Job * feedPublishJob( ParamMap & params, const QString & title, const QString & body,
            const QString & image1, const QString & image1Link,
            const QString & image2, const QString & image2Link,
            const QString & image3, const QString & image3Link,
            const QString & image4, const QString & image4Link, int priority );

    // internals
    class Private;
    Private * d;
}; // Session

} // namespace KFacebook

#endif // KFACEBOOK_SESSION_H
