/*  This file is part of the KDE project
    Copyright (C) 2007,2008 Will Stephenson <wstephenson@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library.  If not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.

*/

#include <QDomDocument>

#include <KDebug>

#include "affiliation.h"
#include "education.h"
#include "highschoolinfo.h"
#include "location.h"
#include "method.h"
#include "userinfo.h"
#include "workinfo.h"

#include "userinfoxmlhelper.h"

QList<KFacebook::UserInfo> UserInfoXMLHelper::parseUsers( const QDomElement & elem )
{
    kDebug() << endl;
    QList<KFacebook::UserInfo> infoList;
    QDomNode n = elem.firstChild();
    while (!n.isNull()) {
        QDomElement e = n.toElement();
        if (!e.isNull()) {
            if ( e.tagName() == "user" ) 
                infoList.append( parseUser( e ) );
        }
        n = n.nextSibling();
    }
    kDebug() << "Parsed UserInfo list." << endl;
    return infoList;
}

KFacebook::UserInfo UserInfoXMLHelper::parseUser( const QDomElement & elem )
{
    kDebug() << endl;
    uint userUid = 0;
    QString aboutMe, activities, birthday, books, firstName, interests, lastName, movies, music, name, quotes, religion, status, tv;
    QList<KFacebook::Affiliation> affiliations;
    KFacebook::Location currentLocation, hometownLocation;
    QList<KFacebook::Education> educationHistory;
    bool hasAddedApp, isAppUser;
    KFacebook::HighschoolInfo highschoolInfo;
    QList<KFacebook::UserInfo::Seeking> lookingFor;
    QList<KFacebook::UserInfo::Gender> interestedIn;
    int notesCount = -1, wallCount = -1;
    KUrl picSmall, picMed, picLarge, picSquare;
    KFacebook::UserInfo::RelationshipStatus relationshipStatus = KFacebook::UserInfo::UnspecifiedRelationshipStatus;
    KFacebook::UserInfo::PoliticalView politics = KFacebook::UserInfo::UnspecifiedPolitics;
    KFacebook::UserInfo::Gender sex = KFacebook::UserInfo::UnspecifiedGender;
    uint significantOtherId = 0;
    QDateTime profileTimestamp, statusTimestamp;
    int timezoneOffset = 0;
    QList<KFacebook::WorkInfo> workHistory;


    QDomNode n = elem.firstChild();
    while (!n.isNull()) {
        QDomElement e = n.toElement();
        if (!e.isNull()) {
            if ( e.tagName() == KFacebook::Method::FIELD_UID )
                userUid = e.text().toUInt();
            else if ( e.tagName() == KFacebook::Method::FIELD_ABOUT_ME ) {
                aboutMe = e.text();
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_ACTIVITIES ) {
                activities = e.text();
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_AFFILIATIONS ) {
                affiliations = parseAffiliations( e );
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_BIRTHDAY ) {
                birthday = e.text();
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_BOOKS ) {
                books = e.text();
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_CURRENT_LOCATION ) {
                currentLocation = parseLocation( e );
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_EDUCATION_HISTORY ) {
                educationHistory = parseEducationHistory( e );
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_FIRST_NAME ) {
                firstName = e.text();
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_HAS_ADDED_APP ) {
                hasAddedApp = ( e.text() == "1" ) ;
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_HOMETOWN_LOCATION ) {
                hometownLocation = parseLocation( e );
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_HIGHSCHOOL_INFO ) {
                highschoolInfo = parseHighschoolInfo( e );
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_INTERESTS ) {
                interests = e.text();
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_IS_APP_USER ) {
                isAppUser = ( e.text() == "1" ) ;
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_LAST_NAME ) {
                lastName = e.text();
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_LOOKING_FOR ) {
                QString text;
                KFacebook::UserInfo::Seeking seeking;
                QDomNode n1 = e.firstChild();
                while (!n1.isNull()) {
                    QDomElement e1 = n1.toElement();
                    if (!e1.isNull()) {
                        if ( e1.tagName() == "meeting_for" ) {
                            text = e1.text();
                            if ( text == "Friendship" ) {
                                seeking = KFacebook::UserInfo::Friendship;
                            } else if ( text == "A Relationship" ) {
                                seeking = KFacebook::UserInfo::Relationship;
                            } else if ( text == "Dating" ) {
                                seeking = KFacebook::UserInfo::Dating;
                            } else if ( text == "Random Play" ) {
                                seeking = KFacebook::UserInfo::RandomPlay;
                            } else if ( text == "Whatever I can get" ) {
                                seeking = KFacebook::UserInfo::Whatever;
                            }
                            lookingFor.append( seeking );
                        }
                    }
                }
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_INTERESTED_IN ) {
                QString genderString;
                KFacebook::UserInfo::Gender gender;
                QDomNode n1 = e.firstChild();
                while (!n1.isNull()) {
                    QDomElement e1 = n1.toElement();
                    if (!e1.isNull()) {
                        if ( e1.tagName() == "sex" ) {
                            genderString = e1.text();
                            if ( genderString == "male" ) {
                                gender = KFacebook::UserInfo::Male;
                            }
                            else if ( genderString == "female" ) {
                                gender = KFacebook::UserInfo::Female;
                            }
                            interestedIn.append( gender );
                        }
                    }
                    n1 = n1.nextSibling();
                }
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_MOVIES ) {
                movies = e.text();
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_MUSIC ) {
                music = e.text();
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_NAME ) {
                name = e.text();
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_NOTES_COUNT ) {
                notesCount = e.text().toUInt();
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_PICTURE_SMALL ) {
                picSmall = KUrl( e.text() );
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_PICTURE_MEDIUM ) {
                picMed = KUrl( e.text() );
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_PICTURE_LARGE ) {
                picLarge = KUrl( e.text() );
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_PICTURE_SQUARE ) {
                picSquare = KUrl( e.text() );
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_POLITICAL ) {
                QString text = e.text();
                if ( text == "Very Liberal" ) {
                    politics = KFacebook::UserInfo::VeryLiberal; 
                } else if ( text == "Liberal" ) {
                    politics = KFacebook::UserInfo::Liberal;
                } else if ( text == "Moderate" ) {
                    politics = KFacebook::UserInfo::Moderate;
                } else if ( text == "Conservative" ) {
                    politics = KFacebook::UserInfo::Conservative;
                } else if ( text == "VeryConservative" ) {
                    politics = KFacebook::UserInfo::VeryConservative;
                } else if ( text == "Apathetic" ) {
                    politics = KFacebook::UserInfo::Apathetic;
                } else if ( text == "Liberation" ) {
                    politics = KFacebook::UserInfo::Liberation;
                } else if ( text == "Other" ) {
                    politics = KFacebook::UserInfo::Other;
                }
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_PROFILE_UPDATE_TIME ) {
                profileTimestamp.setTime_t( e.text().toUInt() );
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_QUOTES ) {
                quotes = e.text();
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_RELATIONSHIP_STATUS) {
                QString text = e.text();
                if ( text == "Single" ) {
                    relationshipStatus = KFacebook::UserInfo::Single; 
                } else if ( text == "In a Relationship" ) {
                    relationshipStatus = KFacebook::UserInfo::InRelationship;
                } else if ( text == "In an Open Relationship" ) {
                    relationshipStatus = KFacebook::UserInfo::InOpenRelationship;
                } else if ( text == "Engaged" ) {
                    relationshipStatus = KFacebook::UserInfo::Engaged;
                } else if ( text == "Married" ) {
                    relationshipStatus = KFacebook::UserInfo::Married;
                } else if ( text == "It's Complicated" ) {
                    relationshipStatus = KFacebook::UserInfo::Complicated;
                }
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_RELIGION ) {
                religion = e.text();
                QString text = e.text();
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_SEX ) {
                QString text = e.text();
                if ( text == "male" ) {
                    sex = KFacebook::UserInfo::Male;
                } else if ( text == "female" ) {
                    sex = KFacebook::UserInfo::Female;
                }
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_SIGNIFICANT_OTHER ) {
                significantOtherId = e.text().toUInt();
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_STATUS ) {
                QDomNode n1 = e.firstChild();
                while (!n1.isNull()) {
                    QDomElement e1 = n1.toElement();
                    if (!e1.isNull()) {
                        if ( e1.tagName() == "message" )
                            status = e1.text();
                        if ( e1.tagName() == "time" )
                            statusTimestamp.setTime_t( e1.text().toInt() );
                    }
                    n1 = n1.nextSibling();
                }
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_TIMEZONE ) {
                timezoneOffset = e.text().toInt();
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_TV ) {
                tv = e.text();
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_WALL_COUNT ) {
                wallCount = e.text().toUInt();
            }
            else if ( e.tagName() == KFacebook::Method::FIELD_WORK_HISTORY ) {
                workHistory = parseWorkHistory( e );
            }
        }
        n = n.nextSibling();
    }
    KFacebook::UserInfo uinfo( userUid, aboutMe, activities, affiliations, birthday, books, currentLocation, educationHistory, firstName, hasAddedApp, highschoolInfo, hometownLocation, lastName, lookingFor, interestedIn, movies, music, name, notesCount, picSmall, picMed, picLarge, picSquare, politics, profileTimestamp, quotes, relationshipStatus, religion, sex, significantOtherId, status, statusTimestamp, timezoneOffset, tv, wallCount, workHistory );
    kDebug() << "Parsed one UserInfo." << endl;
    uinfo.dump();
    return uinfo;
}

QList<KFacebook::Affiliation> UserInfoXMLHelper::parseAffiliations( const QDomElement & elem )
{
    kDebug() << endl;
    QList<KFacebook::Affiliation> affiliations;

    // iterate <affiliation>s
    QDomNode n = elem.firstChild();
    while (!n.isNull()) {
        QDomElement e = n.toElement();
        if (!e.isNull()) {
            QString tag = e.tagName();
            if ( tag == "affiliation" ) {
                // iterate elements within affiliation
                QString name, status;
                uint nid = 0, year = 0;
                KFacebook::Affiliation::Type type;

                QDomNode n1 = e.firstChild();
                while (!n1.isNull()) {
                    QDomElement e1 = n1.toElement();
                    if (!e1.isNull()) {
                        QString tag = e1.tagName();
                        QString text = e1.text();
                        if ( tag == "name" )
                            name = text;
                        else if ( tag == "nid" )
                            nid = text.toUInt();
                        else if ( tag == "year" )
                            year = text.toUInt();
                        else if ( tag == "status" )
                            status = text;
                        else if ( tag == "type" ) {
                            if ( text == "college " ) {
                                type = KFacebook::Affiliation::College;
                            } else if ( text == "high school" ) {
                                type = KFacebook::Affiliation::HighSchool;
                            } else if ( text == "region" ) {
                                type = KFacebook::Affiliation::Region;
                            } else if ( text == "work" ) {
                                type = KFacebook::Affiliation::Work;
                            }
                        }
                    }
                    n1 = n1.nextSibling();
                }
                kDebug() << type << ", " << year << ", " << name << ", " << nid << ", " << status << endl;
                if ( nid != 0 ) {
                    KFacebook::Affiliation aff( type, year, name, nid, status );
                    affiliations.append( aff );
                }
            }
        }
        n = n.nextSibling();
    }
    return affiliations;
}

QList<KFacebook::Education> UserInfoXMLHelper::parseEducationHistory( const QDomElement & elem )
{
    kDebug() << endl;
    QList<KFacebook::Education> educationHistory;
    // iterate <education_info>s
    QDomNode n = elem.firstChild();
    while (!n.isNull()) {
        QDomElement e = n.toElement();
        if (!e.isNull()) {
            QString tag = e.tagName();
            if ( tag == "education_info" ) {
                // iterate elements within education
                QString name;
                QStringList concentrations;
                uint rawYear = 0;

                QDomNode n1 = e.firstChild();
                while (!n1.isNull()) {
                    QDomElement e1 = n1.toElement();
                    if (!e1.isNull()) {
                        QString tag = e1.tagName();
                        QString text = e1.text();
                        if ( tag == "name" )
                            name = text;
                        else if ( tag == "year" )
                            rawYear = text.toUInt();
                        else if ( tag == "concentrations" ) {
                            QDomNode n2 = e1.firstChild();
                            while (!n2.isNull()) {
                                QDomElement e2 = n2.toElement();
                                if (!e2.isNull()) {
                                    if ( e2.tagName() == "concentration" )
                                        concentrations.append( e2.text() );
                                }
                                n2 = n2.nextSibling();
                            }
                        }
                    }
                    n1 = n1.nextSibling();
                }
                kDebug() << name << ", " << rawYear << ", " << concentrations << endl;
                KFacebook::Education edu( QDate(rawYear, 1, 1 ), name, concentrations );
                educationHistory.append( edu );
            }
        }
        n = n.nextSibling();
    }
    return educationHistory;
}

KFacebook::HighschoolInfo UserInfoXMLHelper::parseHighschoolInfo( const QDomElement & elem )
{
    kDebug() << endl;
    QString hs1Name, hs2Name;
    uint hs1Id = 0, hs2Id = 0, year = 0;
    QDomNode n = elem.firstChild();
    while (!n.isNull()) {
        QDomElement e = n.toElement();
        if (!e.isNull()) {
            QString tag = e.tagName();
            QString text = e.text();
            if ( tag == "hs1_name" ) {
                hs1Name = text;
            } else if ( tag == "hs2_name" ) {
                hs2Name = text;
            } else if ( tag == "grad_year" ) {
                year = text.toUInt();
            } else if ( tag == "hs1_id" ) {
                hs1Id = text.toUInt();
            } else if ( tag == "hs2_id" ) {
                hs2Id = text.toUInt();
            }
        }
        n = n.nextSibling();
    }
    QDate gradYear;
    if ( year != 0 ) {
        gradYear.setDate( year, 1, 1 );
    }
    kDebug() << hs1Name << ", " << year << ", " << hs1Id << ", " << hs2Name << ", " << hs2Id << endl;
    KFacebook::HighschoolInfo hsi;
    if ( !( hs1Name.isEmpty() && hs2Name.isEmpty() && ( year == hs1Id == hs2Id == 0 ) ) ) {
        KFacebook::HighschoolInfo valid( hs1Name, gradYear, hs1Id, hs2Name, hs2Id );
        hsi = valid;
    }
    return  hsi;
}

KFacebook::Location UserInfoXMLHelper::parseLocation( const QDomElement & elem )
{
    kDebug() << endl;
    QString city, state, country;
    uint zip = 0;
    QDomNode n = elem.firstChild();
    while (!n.isNull()) {
        QDomElement e = n.toElement();
        if (!e.isNull()) {
            QString tag = e.tagName();
            QString text = e.text();
            if ( tag == "city" ) {
                city = text;
            } else if ( tag == "country" ) {
                country = text;
            } else if ( tag == "state" ) {
                state = text;
            } else if ( tag == "zip" ) {
                zip = text.toUInt();
            }
        }
        n = n.nextSibling();
    }
    kDebug() << city << ", " << country << ", " << state << ", " << zip << endl;
    KFacebook::Location loc;
    if ( !( city.isEmpty() && country.isEmpty() && state.isEmpty() && zip == 0 ) ) {
        KFacebook::Location valid( city, country, state, zip );
        loc = valid;
    }
    return loc;
}

QList<KFacebook::WorkInfo> UserInfoXMLHelper::parseWorkHistory( const QDomElement & elem )
{
    kDebug() << endl;
    QList<KFacebook::WorkInfo> workHistory;

    // iterate <work_info>s
    QDomNode n = elem.firstChild();
    while (!n.isNull()) {
        QDomElement e = n.toElement();
        if (!e.isNull()) {
            QString tag = e.tagName();
            if ( tag == "work_info" ) {
                // iterate elements within affiliation
                QString companyName, position, description;
                QString startDate, endDate;
                KFacebook::Location location;

                QDomNode n1 = e.firstChild();
                while (!n1.isNull()) {
                    QDomElement e1 = n1.toElement();
                    if (!e1.isNull()) {
                        QString tag = e1.tagName();
                        QString text = e1.text();
                        if ( tag == "company_name" )
                            companyName = text;
                        else if ( tag == "position" )
                            position = text;
                        else if ( tag == "description" )
                            description = text;
                        else if ( tag == "start_date" )
                            startDate = text;
                        else if ( tag == "end_date" )
                            endDate = text;
                        else if ( tag == "location" ) {
                            location = parseLocation( e1 );
                        }
                    }
                    n1 = n1.nextSibling();
                }
                kDebug() << companyName << ", " << position << ", " << description << ", " << startDate << ", " << endDate << endl;
                KFacebook::WorkInfo wi( location, companyName, parseDate(startDate), parseDate(endDate), description, position );
                workHistory.append( wi );
            }
        }
        n = n.nextSibling();
    }
    return workHistory;
}

QDate UserInfoXMLHelper::parseDate( const QString & dateString )
{
    // dateString of the form YYYY-MM, YYYY, or MM. It may be blank
    QDate date;
    int year = 0;
    switch ( dateString.length() ) {
        case 2:
            date.setDate( QDate::currentDate().year(), dateString.toInt(), 1 );
            break;
        case 4:
            date.setDate( dateString.toInt(), 1, 1 );
            break;
        case 7:
            year = dateString.mid( 0, 4 ).toInt();
            if ( year == 0 ) {
                year = QDate::currentDate().year();
            }
            date.setDate( year, dateString.mid( 5, 2 ).toInt(), 1 );
            break;
        default:
            break;
    }
    return date;
}
